package es.ucm.fdi.grasia.faerie.demos.talkingAgents.myLocationUpdater;

import es.ucm.fdi.grasia.faerie.core.*;
import es.ucm.fdi.grasia.faerie.core.model.BasicContextValue;
import es.ucm.fdi.grasia.faerie.core.model.IncludedEntities;
import es.ucm.fdi.grasia.faerie.core.util.Functions;
import es.ucm.fdi.grasia.faerie.core.util.ContextURIs;
import es.ucm.fdi.grasia.faerie.demos.talkingAgents.model.WeightDetected;
import es.ucm.fdi.grasia.faerie.demos.talkingAgents.model.WeightSensor;
import es.ucm.fdi.grasia.faerie.demos.talkingAgents.model.MyLocation;
import es.ucm.fdi.grasia.faerie.model.base.RelationalCondition;
import es.ucm.fdi.grasia.faerie.model.base.AttributeConditionSet;
import es.ucm.fdi.grasia.faerie.model.base.Name;
import org.apache.felix.scr.annotations.Component;
import org.apache.log4j.Logger;

import java.util.Map;
import java.net.URI;

/**
 *
 * @author Chema
 */
@Component(inherit = true)
@SuppressWarnings({ "rawtypes", "unchecked" })
public class MyLocationUpdater extends AbstractAttributeUpdaterComponent {

	private AttributeConditionSet<WeightSensor, WeightDetected, Float> activatedSensors = null;

	@Override
	public int handle(ContextEvent event) {
		return Functions.contextElementAddedOrRemoved(MyLocation.class.getName(), event) ? ContextObserver.HANDLE_WRITE : ContextObserver.HANDLE_NONE;
	}

	@Override
	public void elementAdded(Attribute attributeInstance, ContextEvent event) {
		this.putUpdatedAttributeInstance(attributeInstance);

		URI attInstanceId = attributeInstance.getId();

		activatedSensors = new AttributeConditionSet<WeightSensor, WeightDetected, Float>(
				WeightSensor.class, WeightDetected.class);
		activatedSensors.setAttributeCondition(new RelationalCondition(RelationalCondition.GREATER_THAN, Float.valueOf(
				0)));

		this.addAttributeInstanceLinkedElement(attInstanceId, new IncludedEntities<WeightSensor>(activatedSensors));

		this.updateAttributeInstance(attInstanceId);
	}

	@Override
	public synchronized void attributeInstanceLinkedElementsUpdated(Attribute attributeInstance,
			Map<URI, ContextElement> linkedElements) {
		IncludedEntities<WeightSensor> includedActivatedSensors = (IncludedEntities) linkedElements.get(ContextURIs.
				relationshipInstanceURI(IncludedEntities.class, activatedSensors.getId()));

		String result = obtainLastLocation(includedActivatedSensors);

		attributeInstance.update(new BasicContextValue<String>(result, 1.0, this.getBundleEntity()));
	}

	protected synchronized String obtainLastLocation(IncludedEntities<WeightSensor> includedActivatedSensors) {
		int size = includedActivatedSensors.countTargetEntities();
		if (size > 0)
			try {
				WeightSensor sensor = includedActivatedSensors.getTargetEntity(size - 1);
				Name name = sensor.getAttributeInstance(Name.class);
				if (name == null)
					name = this.contextRequest(new Name(sensor));
				ContextValue<String> value = name.getValueWaitIfNull();
				String myLocation = value.getData();
				return LocationAlgorithm.locate(myLocation);
			} catch (InterruptedException ex) {
				Logger.getLogger(this.getClass()).error(ex.getLocalizedMessage(), ex);
			}
		return null;
	}
}